package stranded2.bio;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import stranded2.map.Stranded2Map;
import stranded2.map.io.MapReader;
import stranded2.map.io.MapWriter;
import stranded2.system.S2Item;
import stranded2.system.S2Object;
import stranded2.system.S2Unit;

public class BioGenerator {

	private static int objCounter = 0;
	private static int itemCounter = 0;
	private static int unitCounter = 0;
	private static Map<Integer, Biome> biomes = new HashMap<Integer, Biome>();
	
	private static final int BIO_DECIDUOUS_FOREST = 1;
	private static final int BIO_PLAIN = 2;
	private static final int BIO_SWAMP = 3;
	private static final int BIO_PALM_FOREST = 4;
	private static final int BIO_PALMHEAD_FOREST = 5;
	private static final int BIO_FIR_FOREST = 6;
	private static final int BIO_DESERT = 7;
	private static final int BIO_BANK = 8;
	private static final int BIO_OCEAN = 9;
	private static final int BIO_MOUNTAIN = 10;
	
	public static void main(String[] args) throws IOException {
		
		initializeBiomes();
		
		MapReader reader = new MapReader();
		Stranded2Map map = reader.readMap("C:\\Stranded II\\mods\\Stranded II\\maps\\SomeMap.s2");
		int terrainSize = map.getHeightmap().getTerrainSize();
		BioMap bioMap = new BioMap("C:\\Stranded II\\mods\\Stranded II\\biomaps\\biomap.bmp",terrainSize+1, terrainSize+1);
		int lowLimit = -(terrainSize*64)/2;
		if (!map.getObjects().isEmpty()) throw new RuntimeException();
		for (int x=0; x<=terrainSize; x++)
			for (int y=0; y<=terrainSize; y++) {
				Biome biome = getBiomeForCoords(x,y,bioMap);
				if (biome!=null) {
					S2Object randomObject = biome.getRandomObject();
					if (randomObject!=null) {
						randomObject.setDayTimer(0);
						randomObject.setHealth(1000f);
						randomObject.setHealthMax(1000f);
						randomObject.setYaw((float) (Math.random()*360d));
						randomObject.setX((float) (lowLimit + x*64 + 4 + Math.random()*56d));
						randomObject.setZ((float) (lowLimit + y*64 + 4 + Math.random()*56d));
						float height = map.getHeightmap().getHeight(x,y);
						if (height > biome.getLowerLimit() && height < biome.getUpperLimit()) {
							randomObject.setId(++objCounter);
							map.getObjects().add(randomObject);
						}
					}
					S2Item randomItem = biome.getRandomItem();
					if (randomItem!=null) {
						randomItem.setCount(1);
						randomItem.setParentClass((byte) 0);
						randomItem.setParentId(0);
						randomItem.setParentMode((byte) 0);
						randomItem.setHealth(1000f);
						randomItem.setYaw((float) (Math.random()*360d));
						randomItem.setX((float) (lowLimit + x*64 + 4 + Math.random()*56d));
						randomItem.setY(0f);
						randomItem.setZ((float) (lowLimit + y*64 + 4 + Math.random()*56d));
						float height = map.getHeightmap().getHeight(x,y);
						if (height > biome.getLowerLimit() && height < biome.getUpperLimit()) {
							randomItem.setId(++itemCounter);
							map.getItems().add(randomItem);
						}
					}
					S2Unit randomUnit = biome.getRandomUnit();
					if (randomUnit!=null) {
						randomUnit.setHealth(1000f);
						randomUnit.setHealthMax(1000f);
						randomUnit.setExhaustion(0f);
						randomUnit.setHunger(0f);
						randomUnit.setThirst(0f);
						randomUnit.setYaw((float) (Math.random()*360d));
						randomUnit.setX((float) (lowLimit + x*64 + 4 + Math.random()*56d));
						randomUnit.setY(0f);
						randomUnit.setZ((float) (lowLimit + y*64 + 4 + Math.random()*56d));
						randomUnit.setAicx(randomUnit.getX());
						randomUnit.setAicz(randomUnit.getZ());
						float height = map.getHeightmap().getHeight(x,y);
						if (height > biome.getLowerLimit() && height < biome.getUpperLimit()) {
							randomUnit.setId(++unitCounter);
							map.getUnits().add(randomUnit);
						}
					}
				}
			}
		
		MapWriter writer = new MapWriter();
		writer.writeMap(map, "C:\\Stranded II\\mods\\Stranded II\\maps\\SomeMap-biomes.s2");
	}

	private static Biome getBiomeForCoords(int x, int y, BioMap bioMap) {
		char[] rgb = bioMap.getRGB(x, y);
		for (Iterator<Integer> keyIter = biomes.keySet().iterator(); keyIter.hasNext();) {
			Biome biome = biomes.get(keyIter.next());
			if (rgb[0]==biome.getR()&&rgb[1]==biome.getG()&&rgb[2]==biome.getB())
				return biome;
		}
		return null;
	}

	private static void initializeBiomes() {
		// Fort de feuillus
		Biome foretDeFeuillus = new Biome(132f/256f,144f/256f);
		foretDeFeuillus.setR(0);
		foretDeFeuillus.setG(128);
		foretDeFeuillus.setB(0);
		//10 - petit feuillu
		foretDeFeuillus.addObject(10, 0.005f);
		//12 - petit tronc fin
		foretDeFeuillus.addObject(12, 0.01f);
		//13 - grand tronc fin
		foretDeFeuillus.addObject(13, 0.003f);
		//14 - feuillu moyen
		foretDeFeuillus.addObject(14, 0.005f);
		//15 - triple troncs fins
		foretDeFeuillus.addObject(15, 0.005f);
		//16 - grand feuillu
		foretDeFeuillus.addObject(16, 0.005f);
		//18 - arbre  cacao
		foretDeFeuillus.addObject(18, 0.0001f);
		//19 - arbre  caf
		foretDeFeuillus.addObject(19, 0.0001f);
		//195 - arbre  prunes
		foretDeFeuillus.addObject(195, 0.0001f);

		//33 - fougre
		foretDeFeuillus.addObject(33, 0.015f);
		//34 - petit buisson
		foretDeFeuillus.addObject(34, 0.015f);
		//38 - fougre feuilles brunes
		foretDeFeuillus.addObject(38, 0.015f);
		//41 - gros buisson
		foretDeFeuillus.addObject(41, 0.015f);
		//42 - petit buisson
		foretDeFeuillus.addObject(42, 0.015f);
		//46 - buisson  baies
		foretDeFeuillus.addObject(46, 0.005f);

		//50 - petit carr d'herbes
		foretDeFeuillus.addObject(50, 0.015f);
		//51 - carr moyen d'herbes
		foretDeFeuillus.addObject(51, 0.01f);
		//52 - gros carr d'herbes
		foretDeFeuillus.addObject(52, 0.005f);

		//106 - fleurs
		foretDeFeuillus.addObject(106, 0.005f);
		//107 - fleurs
		foretDeFeuillus.addObject(107, 0.005f);
		//108 - fleurs
		foretDeFeuillus.addObject(108, 0.005f);
		//109 - fleurs
		foretDeFeuillus.addObject(109, 0.005f);

		//i31 - petit champignon
		foretDeFeuillus.addItem(31, 0.005f);
		//i32 - gros champignon
		foretDeFeuillus.addItem(32, 0.005f);
		//i35 - champignon rouge
		foretDeFeuillus.addItem(35, 0.005f);
		//i17 - herbe soin
		foretDeFeuillus.addItem(17, 0.003f);

		//202 - mousse
		foretDeFeuillus.addObject(202, 0.005f);
		//130 - racines
		foretDeFeuillus.addObject(130, 0.005f);
		//131 - racines
		foretDeFeuillus.addObject(131, 0.005f);
		//132 - souche
		foretDeFeuillus.addObject(132, 0.0005f);
		//135 - longue souche
		foretDeFeuillus.addObject(135, 0.0005f);

		//u16 - libellule
		foretDeFeuillus.addUnit(16, 0.02f);
		//u20 - araigne
		foretDeFeuillus.addUnit(20, 0.02f);
		//u26 - kiwi
		foretDeFeuillus.addUnit(26, 0.02f);
		
		biomes.put(BIO_DECIDUOUS_FOREST, foretDeFeuillus);
		
		//Plaine
		Biome plaine = new Biome(132f/256f,154f/256f);
		plaine.setR(0);
		plaine.setG(255);
		plaine.setB(0);
		//10 - petit feuillu
		plaine.addObject(10, 0.005f);

		//31 - grosses feuilles, 3
		plaine.addObject(31, 0.025f);
		//32 - grosses feuilles, 6
		plaine.addObject(32, 0.015f);
		//34 - petit buisson
		plaine.addObject(34, 0.025f);
		//35 - blue blossom
		plaine.addObject(35, 0.005f);
		//37 - grosses feuilles, 3 abimes
		plaine.addObject(37, 0.025f);

		//52 - gros carr d'herbes
		plaine.addObject(52, 0.045f);
		//53 - champs
		plaine.addObject(53, 0.045f);

		//100 - fleurs
		plaine.addObject(100, 0.015f);
		//101 - fleurs
		plaine.addObject(101, 0.015f);
		//102 - fleurs
		plaine.addObject(102, 0.015f);
		//103 - fleurs
		plaine.addObject(103, 0.015f);
		//104 - fleurs
		plaine.addObject(104, 0.015f);
		//105 - fleurs
		plaine.addObject(105, 0.015f);
		//106 - fleurs
		plaine.addObject(106, 0.015f);
		//107 - fleurs
		plaine.addObject(107, 0.015f);
		//108 - fleurs
		plaine.addObject(108, 0.015f);
		//109 - fleurs
		plaine.addObject(109, 0.015f);
		//110 - fleur poison
		plaine.addObject(110, 0.005f);

		//i33 - champignon brun
		plaine.addItem(33, 0.025f);
		//i18 - herbe soin 2
		plaine.addItem(18, 0.002f);
		//i19 - herbe dore
		plaine.addItem(19, 0.002f);

		//44 - pied de vigne
		plaine.addObject(44, 0.005f);
		//47 - plan de cotton
		plaine.addObject(47, 0.015f);

		//121 - grains
		plaine.addObject(121, 0.015f);

		//134 - plant de canabis
		plaine.addObject(134, 0.005f);
		
		//u6 - papillon 1
		plaine.addUnit(6, 0.03f);
		//u7 - papillon 2
		plaine.addUnit(7, 0.03f);
		//u15 - mouton
		plaine.addUnit(15, 0.03f);
		//u17 - mouche
		plaine.addUnit(17, 0.02f);
		//u16 - libellule
		plaine.addUnit(16, 0.02f);
		//u21 - luciole
		plaine.addUnit(21, 0.02f);
		//u25 - oiseau
		plaine.addUnit(25, 0.02f);

		biomes.put(BIO_PLAIN, plaine);
		
		//Fort de marais
		Biome marais = new Biome(132f/256f,144f/256f);
		marais.setR(128);
		marais.setG(128);
		marais.setB(0);
		//11 - tronc  lianes nu
		marais.addObject(11, 0.02f);
		//17 - tronc  lianes
		marais.addObject(17, 0.015f);
		//185 - longues feuilles
		marais.addObject(185, 0.05f);
		//50 - petit carr d'herbes
		marais.addObject(50, 0.05f);
		//i34 - champignon poison
		marais.addItem(34, 0.005f);
		//i20 - racine aigre
		marais.addItem(20, 0.005f);
//		30 - buisson mort
		marais.addObject(30, 0.005f);
		
		biomes.put(BIO_SWAMP, marais);

		//Fort de palmiers
		Biome foretDePalmiers = new Biome(132f/256f,144f/256f);
		foretDePalmiers.setR(255);
		foretDePalmiers.setG(255);
		foretDePalmiers.setB(0);
		//1 - palmier pench
		foretDePalmiers.addObject(1, 0.008f);
		//2 - palmier un peu pench
		foretDePalmiers.addObject(2, 0.008f);
		//3 - palmier droit
		foretDePalmiers.addObject(3, 0.008f);
		//4 - double palmier
		foretDePalmiers.addObject(4, 0.004f);
		//5 - palmier pli
		foretDePalmiers.addObject(5, 0.002f);
		//7 - grand palmier
		foretDePalmiers.addObject(7, 0.007f);
		//8 - bananier
		foretDePalmiers.addObject(8, 0.005f);
		//9 - cocotier
		foretDePalmiers.addObject(9, 0.005f);
		//213 - palmier  lianes
		foretDePalmiers.addObject(213, 0.005f);

		//36 - buisson palmier fin
		foretDePalmiers.addObject(36, 0.005f);
		//45 - buisson palmier
		foretDePalmiers.addObject(45, 0.005f);

		//133 - souche de palmier
		foretDePalmiers.addObject(133, 0.005f);
		
		//i102 - coquillage
		foretDePalmiers.addItem(102, 0.003f);

		//u3 - crabe
		foretDePalmiers.addUnit(3, 0.03f);
		//u10 - tortue
		foretDePalmiers.addUnit(10, 0.02f);
		//u36 - perroquet
		foretDePalmiers.addUnit(36, 0.02f);
		//u25 - oiseau
		foretDePalmiers.addUnit(25, 0.02f);

		biomes.put(BIO_PALM_FOREST, foretDePalmiers);

//		Fort de palmiers plumeau
		Biome foretDePalmiersPlumeaux = new Biome(132f/256f,144f/256f);
		foretDePalmiersPlumeaux.setR(255);
		foretDePalmiersPlumeaux.setG(255);
		foretDePalmiersPlumeaux.setB(128);
//		6 - palmier plumeau
		foretDePalmiersPlumeaux.addObject(6, 0.05f);
//		39 - pied de palmier plumeau
		foretDePalmiersPlumeaux.addObject(39, 0.15f);
//		40 - buisson  feuilles larges
		foretDePalmiersPlumeaux.addObject(40, 0.1f);
		
		//u3 - crabe
		foretDePalmiersPlumeaux.addUnit(3, 0.03f);
		//u10 - tortue
		foretDePalmiersPlumeaux.addUnit(10, 0.02f);
		//u25 - oiseau
		foretDePalmiersPlumeaux.addUnit(25, 0.02f);

		biomes.put(BIO_PALMHEAD_FOREST, foretDePalmiersPlumeaux);

//		Fort de sapins
		Biome foretDeSapins = new Biome(132f/256f,158f/256f);
		foretDeSapins.setR(0);
		foretDeSapins.setG(64);
		foretDeSapins.setB(0);
//		20 - petit sapin
		foretDeSapins.addObject(20, 0.30f);
//		21 - grand sapin
		foretDeSapins.addObject(21, 0.10f);
		//33 - fougre
		foretDeSapins.addObject(33, 0.06f);
		//34 - petit buisson
		foretDeSapins.addObject(34, 0.06f);
		//38 - fougre feuilles brunes
		foretDeSapins.addObject(38, 0.06f);
		//41 - gros buisson
		foretDeSapins.addObject(41, 0.06f);
		//50 - petit carr d'herbes
		foretDeSapins.addObject(50, 0.04f);
		//51 - carr moyen d'herbes
		foretDeSapins.addObject(51, 0.04f);
		//52 - gros carr d'herbes
		foretDeSapins.addObject(52, 0.04f);
		//u25 - oiseau
		foretDeSapins.addUnit(25, 0.02f);

		
		biomes.put(BIO_FIR_FOREST, foretDeSapins);
		
//		Dsert
		Biome desert = new Biome(132f/256f,180f/256f);
		desert.setR(32);
		desert.setG(32);
		desert.setB(64);
//		30 - buisson mort
		desert.addObject(30, 0.08f);
		//132 - souche
		desert.addObject(132, 0.01f);
		//135 - longue souche
		desert.addObject(135, 0.01f);

		//u14 - scorpion
		desert.addUnit(14, 0.02f);
		
		biomes.put(BIO_DESERT, desert);
		
//		Bords de plan d'eau
		Biome bordDePlanDEau = new Biome(124f/256f,128f/256f);
		bordDePlanDEau.setR(0);
		bordDePlanDEau.setG(255);
		bordDePlanDEau.setB(255);
//		43 - roseau
		bordDePlanDEau.addObject(43, 0.4f);
//		120 - bambou
		bordDePlanDEau.addObject(120, 0.25f);

		//i102 - coquillage
		bordDePlanDEau.addItem(102, 0.03f);
		//i103 - etoile de mer
		bordDePlanDEau.addItem(103, 0.03f);
		//i109 - algues
		bordDePlanDEau.addItem(109, 0.03f);

		//u3 - crabe
		bordDePlanDEau.addUnit(3, 0.05f);
		//u10 - tortue
		bordDePlanDEau.addUnit(10, 0.02f);
		//u16 - libellule
		bordDePlanDEau.addUnit(16, 0.015f);

		biomes.put(BIO_BANK, bordDePlanDEau);
		
//		Ocan
		Biome ocean = new Biome(0f,124f/256f);
		ocean.setR(0);
		ocean.setG(0);
		ocean.setB(255);
//		48 - algues
		ocean.addObject(48, 0.35f);
//		210 - corail		
		ocean.addObject(210, 0.10f);

		//i102 - coquillage
		ocean.addItem(102, 0.08f);
		//i103 - etoile de mer
		ocean.addItem(103, 0.05f);
		//i109 - algues
		ocean.addItem(109, 0.08f);

		//u4 - poisson
		ocean.addUnit(4, 0.01f);
		//u11 - poisson
		ocean.addUnit(11, 0.01f);
		//u12 - poisson
		ocean.addUnit(12, 0.01f);

		biomes.put(BIO_OCEAN, ocean);
		
		//Montagne, rochers
		Biome montagne = new Biome(144f/256f,1f);
		montagne.setR(192);
		montagne.setG(192);
		montagne.setB(255);

		//petit rocher
		montagne.addObject(63, 0.08f);
		//gros rocher
		montagne.addObject(64, 0.02f);
		//rocher en pointe
		montagne.addObject(65, 0.05f);
		//double rocher
		montagne.addObject(67, 0.02f);
		//gros double rocher
		montagne.addObject(68, 0.02f);
		//gros rocher pointu
		montagne.addObject(68, 0.02f);
		//metal
		montagne.addObject(70, 0.005f);
		//or
		montagne.addObject(71, 0.002f);
		
		//eau plein
		montagne.addObject(73, 0.004f);
		//eau vide
		montagne.addObject(74, 0.004f);
		//fontaine
		montagne.addObject(75, 0.003f);
		
		//cailloux
		montagne.addItem(21, 0.02f);
		//petit rock
		montagne.addItem(22, 0.02f);
		//pierre  feu
		montagne.addItem(23, 0.005f);
		
		biomes.put(BIO_MOUNTAIN, montagne);
	}
}
