package stranded2.bio;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import stranded2.system.S2Item;
import stranded2.system.S2Object;
import stranded2.system.S2Unit;

public class Biome {

	private Map<Byte, Float> objectProbabilities = new HashMap<Byte, Float>();
	private Map<Byte, Float> itemProbabilities = new HashMap<Byte, Float>();
	private Map<Byte, Float> unitProbabilities = new HashMap<Byte, Float>();

	private float lowerLimit;
	private float upperLimit;
	private int r;
	private int g;
	private int b;

	public Biome(float low, float high) {
		this.lowerLimit = low;
		this.upperLimit = high;
	}

	public void addObject(int i, float f) {
		objectProbabilities.put((byte) i, f);
	}

	public float getObject(byte i) {
		return objectProbabilities.get(i);
	}

	public void addItem(int i, float f) {
		itemProbabilities.put((byte) i, f);
	}

	public float getItem(byte i) {
		return itemProbabilities.get(i);
	}

	public void addUnit(int i, float f) {
		unitProbabilities.put((byte) i, f);
	}

	public float getUnit(byte i) {
		return unitProbabilities.get(i);
	}

	private Byte getRandomFromMap(Map<Byte, Float> map) {
		float rand = (float) Math.random();
		float currentVal = 0f;
		boolean atEnd = false;
		for (Iterator<Byte> i = map.keySet().iterator(); i.hasNext() && !atEnd;) {
			Byte currentKey = i.next();
			Float currentProbability = map.get(currentKey);
			currentVal += currentProbability;
			if (rand < currentVal) {
				return currentKey;
			}
		}
		return null;
	}

	public S2Object getRandomObject() {
		S2Object object = null;
		Byte type = getRandomFromMap(objectProbabilities);
		if (type != null) {
			object = new S2Object();
			object.setType(type);
		}
		return object;
	}

	public S2Item getRandomItem() {
		S2Item object = null;
		Byte type = getRandomFromMap(itemProbabilities);
		if (type != null) {
			object = new S2Item();
			object.setType(type);
		}
		return object;
	}

	public S2Unit getRandomUnit() {
		S2Unit object = null;
		Byte type = getRandomFromMap(unitProbabilities);
		if (type != null) {
			object = new S2Unit();
			object.setType(type);
		}
		return object;
	}

	public float getLowerLimit() {
		return lowerLimit;
	}

	public void setLowerLimit(float lowerLimit) {
		this.lowerLimit = lowerLimit;
	}

	public float getUpperLimit() {
		return upperLimit;
	}

	public void setUpperLimit(float upperLimit) {
		this.upperLimit = upperLimit;
	}

	public int getR() {
		return r;
	}

	public void setR(int r) {
		this.r = r;
	}

	public int getG() {
		return g;
	}

	public void setG(int g) {
		this.g = g;
	}

	public int getB() {
		return b;
	}

	public void setB(int b) {
		this.b = b;
	}

}
