package stranded2.map.io;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;

public class FloatImage {

	private BufferedImage image;
	private byte[] data;
	private float[][] floatData;

	public FloatImage(SimpleReader reader, int width, int height) throws IOException {
		image = new BufferedImage(width + 1, height + 1, BufferedImage.TYPE_INT_BGR);

		byte[] byteTab = new byte[(height + 1) * (width + 1) * 4];
		floatData = new float[height+1][width+1];
		ByteBuffer byteBuffer = ByteBuffer.wrap(byteTab);
		byteBuffer.order(ByteOrder.LITTLE_ENDIAN);

		for (int x = 0; x <= width; x++) {
			for (int y = 0; y <= height; y++) {
				float val = floatData[y][x] = reader.readFloat();
				byteBuffer.putFloat(val);
				int r = (int) (val * 256);
				int rgb = 255 * 256 * 256 * 256 + r * 256 * 256 + r * 256 + r;
				setImageData(x, y, rgb);
			}
		}
		data = byteBuffer.array();

	}

	public float getHeight(int x, int z) {
		return floatData[z][x];
	}

	public byte[] getData() {
		return data;
	}

	public void setImageData(int x, int y, int rgb) {
		image.setRGB(x, y, rgb);
	}

	public void createBitmap(String path) throws IOException {
		Iterator<ImageWriter> imageWritersByFormatName = ImageIO.getImageWritersByFormatName("bmp");
		ImageWriter writer = imageWritersByFormatName.next();
		ImageOutputStream imageOutputStream = ImageIO.createImageOutputStream(new File(path));
		writer.setOutput(imageOutputStream);
		writer.write(image);
		imageOutputStream.close();
	}
}
