package stranded2.map.io;

import java.io.IOException;

import stranded2.map.S2MapColormap;
import stranded2.map.S2MapExtensions;
import stranded2.map.S2MapGrassmap;
import stranded2.map.S2MapHeader;
import stranded2.map.S2MapHeightmap;
import stranded2.map.S2MapImage;
import stranded2.map.S2MapInfos;
import stranded2.map.S2MapInterface;
import stranded2.map.S2MapItems;
import stranded2.map.S2MapObjects;
import stranded2.map.S2MapPasswordHeader;
import stranded2.map.S2MapStates;
import stranded2.map.S2MapUnits;
import stranded2.map.S2MapVariables;
import stranded2.map.Stranded2Map;
import stranded2.system.S2Extension;
import stranded2.system.S2Info;
import stranded2.system.S2Item;
import stranded2.system.S2Object;
import stranded2.system.S2State;
import stranded2.system.S2Unit;

public class MapReader {

	private SimpleReader reader;
	private String path;

	public Stranded2Map readMap(String path) throws IOException {

		this.path = path;

		Stranded2Map map = new Stranded2Map();
		reader = new SimpleReader(path);

		map.setHeader(readHeader());
		map.setImage(readImage());
		map.getImage().createBitmap("C:\\Stranded II\\mods\\Stranded II\\biomaps\\image.bmp");
		map.setPasswordHeader(readPasswordHeader());
		map.setMapVariables(readMapVariables());
		map.setMapInterface(readMapInterface());
		map.setColormap(readColormap());
		map.getColormap().createBitmap("C:\\Stranded II\\mods\\Stranded II\\biomaps\\colormap.bmp");
		map.setHeightmap(readHeightmap());
		map.getHeightmap().createBitmap("C:\\Stranded II\\mods\\Stranded II\\biomaps\\heightmap.bmp");
		map.setGrassmap(readGrassmap(map.getColormap().getSize()));
		map.getGrassmap().createBitmap("C:\\Stranded II\\mods\\Stranded II\\biomaps\\grassmap.bmp");
		map.setObjects(readObjects());
		map.setUnits(readUnits());
		map.setItems(readItems());
		map.setInfos(readInfos());
		map.setStates(readStates());
		map.setExtensions(readExtensions());

		reader.readLine();
		reader.readLine();
		String finalLine = reader.readLine();
		System.out.println(finalLine);

		return map;
	}

	private S2MapExtensions readExtensions() throws IOException {
		S2MapExtensions extensions = new S2MapExtensions();
		int extCount = reader.readInt();
		for (int i = 1; i <= extCount; i++) {
			extensions.add(readNextExtension());
		}
		return extensions;
	}

	private S2Extension readNextExtension() throws IOException {
		S2Extension extension = new S2Extension();
		extension.setType(reader.readByte());
		extension.setParentClass(reader.readByte());
		extension.setParentId(reader.readInt());
		extension.setMode(reader.readInt());
		extension.setKey(reader.readString());
		extension.setValue(reader.readString());
		extension.setStuff(reader.readString());
		return extension;
	}

	private S2MapStates readStates() throws IOException {
		S2MapStates mapStates = new S2MapStates();
		int stateCount = reader.readInt();
		for (int i = 1; i <= stateCount; i++) {
			mapStates.add(readNextState());
		}
		return mapStates;
	}

	private S2State readNextState() throws IOException {
		S2State state = new S2State();
		state.setType(reader.readByte());
		state.setParentClass(reader.readByte());
		state.setParentId(reader.readInt());
		state.setX(reader.readFloat());
		state.setY(reader.readFloat());
		state.setZ(reader.readFloat());
		state.setfX(reader.readFloat());
		state.setfY(reader.readFloat());
		state.setFz(reader.readFloat());
		state.setValue(reader.readInt());
		state.setValueFloat(reader.readFloat());
		state.setValueString(reader.readString());
		return state;
	}

	private S2MapInfos readInfos() throws IOException {
		S2MapInfos mapInfos = new S2MapInfos();
		int infoCount = reader.readInt();
		for (int i = 1; i <= infoCount; i++) {
			mapInfos.add(readNextInfo());
		}
		return mapInfos;
	}

	private S2Info readNextInfo() throws IOException {
		S2Info info = new S2Info();
		info.setId(reader.readInt());
		info.setType(reader.readByte());
		info.setX(reader.readFloat());
		info.setY(reader.readFloat());
		info.setZ(reader.readFloat());
		info.setPitch(reader.readFloat());
		info.setYaw(reader.readFloat());
		info.setVars(reader.readString());
		return info;
	}

	private S2MapItems readItems() throws IOException {
		S2MapItems mapItems = new S2MapItems();
		int itemCount = reader.readInt();
		for (int i = 1; i <= itemCount; i++) {
			mapItems.add(readNextItem());
		}
		return mapItems;
	}

	private S2Item readNextItem() throws IOException {
		S2Item item = new S2Item();
		item.setId(reader.readInt());
		item.setType(reader.readByte());
		item.setX(reader.readFloat());
		item.setY(reader.readFloat());
		item.setZ(reader.readFloat());
		item.setYaw(reader.readFloat());
		item.setHealth(reader.readFloat());
		item.setCount(reader.readInt());
		item.setParentClass(reader.readByte());
		item.setParentMode(reader.readByte());
		item.setParentId(reader.readInt());
		return item;
	}

	private S2MapUnits readUnits() throws IOException {
		S2MapUnits mapUnits = new S2MapUnits();
		int objectCount = reader.readInt();
		for (int i = 1; i <= objectCount; i++) {
			mapUnits.add(readNextUnit());
		}
		return mapUnits;
	}

	private S2Unit readNextUnit() throws IOException {
		S2Unit unit = new S2Unit();
		unit.setId(reader.readInt());
		unit.setType(reader.readByte());
		unit.setX(reader.readFloat());
		unit.setY(reader.readFloat());
		unit.setZ(reader.readFloat());
		unit.setYaw(reader.readFloat());
		unit.setHealth(reader.readFloat());
		unit.setHealthMax(reader.readFloat());
		unit.setHunger(reader.readFloat());
		unit.setThirst(reader.readFloat());
		unit.setExhaustion(reader.readFloat());
		unit.setAicx(reader.readFloat());
		unit.setAicz(reader.readFloat());
		return unit;
	}

	private S2MapObjects readObjects() throws IOException {
		S2MapObjects mapObjects = new S2MapObjects();

		int objectCount = reader.readInt();
		for (int i = 1; i <= objectCount; i++) {
			mapObjects.add(readNextObject());
		}
		return mapObjects;
	}

	private S2Object readNextObject() throws IOException {
		S2Object object = new S2Object();
		object.setId(reader.readInt());
		object.setType(reader.readByte());
		object.setX(reader.readFloat());
		object.setZ(reader.readFloat());
		object.setYaw(reader.readFloat());
		object.setHealth(reader.readFloat());
		object.setHealthMax(reader.readFloat());
		object.setDayTimer(reader.readInt());
		return object;
	}

	private S2MapGrassmap readGrassmap(int size) throws IOException {
		S2MapGrassmap grassmap = new S2MapGrassmap(reader, size);
		return grassmap;
	}

	private S2MapHeightmap readHeightmap() throws IOException {
		int terrainSize = reader.readInt();
		if (terrainSize < 0)
			terrainSize = 32;
		S2MapHeightmap heightmap = new S2MapHeightmap(reader, terrainSize);
		return heightmap;
	}

	private S2MapColormap readColormap() throws IOException {
		S2MapColormap colormap = new S2MapColormap(reader, reader.readInt());
		return colormap;
	}

	private S2MapInterface readMapInterface() throws IOException {
		S2MapInterface mapInterface = new S2MapInterface();

		for (int i = 0; i < 10; i++)
			mapInterface.setQuickslot(i, reader.readString());

		return mapInterface;
	}

	private S2MapVariables readMapVariables() throws IOException {
		S2MapVariables variables = new S2MapVariables();

		variables.setName(path);
		variables.setPath(path);
		variables.setTimeDay(reader.readInt());
		variables.setTimeHour(reader.readByte());
		variables.setTimeMinute(reader.readByte());
		variables.setTimeFrozen(reader.readByte());
		variables.setSkybox(reader.readString());
		variables.setMultiplayer(reader.readByte());
		variables.setClimate(reader.readByte());
		variables.setMusic(reader.readString());
		variables.setBriefing(reader.readString());
		variables.setFogR(reader.readByte());
		variables.setFogG(reader.readByte());
		variables.setFogB(reader.readByte());
		variables.setFogMode(reader.readByte());
		variables.setExtendedStuff(reader.readByte());

		return variables;
	}

	private S2MapPasswordHeader readPasswordHeader() throws IOException {
		S2MapPasswordHeader passwordHeader = new S2MapPasswordHeader();
		passwordHeader.setCode(reader.readByte());
		passwordHeader.setPassword(reader.readLine());
		System.out.println(passwordHeader.getPassword());
		return passwordHeader;
	}

	private S2MapImage readImage() throws IOException {
		S2MapImage image = new S2MapImage(reader);
		return image;
	}

	private S2MapHeader readHeader() throws IOException {
		S2MapHeader header = new S2MapHeader();
		header.setFirstLine(reader.readLine());
		header.setVersion(reader.readLine());
		header.setDate(reader.readLine());
		header.setTime(reader.readLine());
		header.setFormat(reader.readLine());
		header.setMode(reader.readLine());

		String typeFormatInfo = reader.readLine();
		if (!"".equals(typeFormatInfo)) {
			header.setTypeFormatItem(Integer.parseInt(typeFormatInfo.substring(0, 1)));
			header.setTypeFormatObject(Integer.parseInt(typeFormatInfo.substring(1, 2)));
			header.setTypeFormatUnit(Integer.parseInt(typeFormatInfo.substring(2, 3)));
		}

		reader.readLine();
		reader.readLine();
		reader.readLine();
		reader.readLine();
		String headerEndLine = reader.readLine();
		System.out.println(headerEndLine);

		return header;
	}
}
