package stranded2.map.io;

import java.io.IOException;
import java.util.Iterator;

import stranded2.map.S2MapColormap;
import stranded2.map.S2MapExtensions;
import stranded2.map.S2MapGrassmap;
import stranded2.map.S2MapHeader;
import stranded2.map.S2MapHeightmap;
import stranded2.map.S2MapImage;
import stranded2.map.S2MapInfos;
import stranded2.map.S2MapInterface;
import stranded2.map.S2MapItems;
import stranded2.map.S2MapObjects;
import stranded2.map.S2MapPasswordHeader;
import stranded2.map.S2MapStates;
import stranded2.map.S2MapUnits;
import stranded2.map.S2MapVariables;
import stranded2.map.Stranded2Map;
import stranded2.system.S2Extension;
import stranded2.system.S2Info;
import stranded2.system.S2Item;
import stranded2.system.S2Object;
import stranded2.system.S2State;
import stranded2.system.S2Unit;

public class MapWriter {

	private SimpleWriter writer;

	public void writeMap(Stranded2Map map, String path) throws IOException {

		writer = new SimpleWriter(path);

		writeHeader(map.getHeader());
		writeImage(map.getImage());
		writePasswordHeader(map.getPasswordHeader());
		writeMapVariables(map.getMapVariables());
		writeMapInterface(map.getMapInterface());
		writeColormap(map.getColormap());
		writeHeightmap(map.getHeightmap());
		writeGrassmap(map.getGrassmap());
		writeObjects(map.getObjects());
		writeUnits(map.getUnits());
		writeItems(map.getItems());
		writeInfos(map.getInfos());
		writeStates(map.getStates());
		writeExtensions(map.getExtensions());
		writer.writeLine(null);
		writer.writeLine("### EOF Map File");
		writer.writeLine("www.unrealsoftware.de");
		
		writer.close();
	}

	private void writeExtensions(S2MapExtensions mapExtensions) throws IOException {
		writer.writeInt(mapExtensions.getCount());
		for (Iterator<S2Extension> i = mapExtensions.getIterator(); i.hasNext();) {
			S2Extension s2Extension = i.next();
			writeNextExtension(s2Extension);
		}
	}

	private void writeNextExtension(S2Extension extension) throws IOException {
		writer.writeByte(extension.getType());
		writer.writeByte(extension.getParentClass());
		writer.writeInt(extension.getParentId());
		writer.writeInt(extension.getMode());
		writer.writeString(extension.getKey());
		writer.writeString(extension.getValue());
		writer.writeString(extension.getStuff());
	}

	private void writeStates(S2MapStates mapStates) throws IOException {
		writer.writeInt(mapStates.getCount());
		for (Iterator<S2State> i = mapStates.getIterator(); i.hasNext();) {
			S2State s2State = i.next();
			writeNextState(s2State);
		}
	}

	private void writeNextState(S2State state) throws IOException {
		writer.writeByte(state.getType());
		writer.writeByte(state.getParentClass());
		writer.writeInt(state.getParentId());
		writer.writeFloat(state.getX());
		writer.writeFloat(state.getY());
		writer.writeFloat(state.getZ());
		writer.writeFloat(state.getfX());
		writer.writeFloat(state.getfY());
		writer.writeFloat(state.getFz());
		writer.writeInt(state.getValue());
		writer.writeFloat(state.getValueFloat());
		writer.writeString(state.getValueString());
	}

	private void writeInfos(S2MapInfos mapInfos) throws IOException {
		writer.writeInt(mapInfos.getCount());
		for (Iterator<Integer> i = mapInfos.getKeyIterator(); i.hasNext();) {
			S2Info s2Info = mapInfos.get(i.next());
			writeNextInfo(s2Info);
		}
	}

	private void writeNextInfo(S2Info info) throws IOException {
		writer.writeInt(info.getId());
		writer.writeByte(info.getType());
		writer.writeFloat(info.getX());
		writer.writeFloat(info.getY());
		writer.writeFloat(info.getZ());
		writer.writeFloat(info.getPitch());
		writer.writeFloat(info.getYaw());
		writer.writeString(info.getVars());
	}

	private void writeItems(S2MapItems mapItems) throws IOException {
		writer.writeInt(mapItems.getCount());
		for (Iterator<Integer> i = mapItems.getKeyIterator(); i.hasNext();) {
			S2Item s2Item = mapItems.get(i.next());
			writeNextItem(s2Item);
		}
	}

	private void writeNextItem(S2Item item) throws IOException {
		writer.writeInt(item.getId());
		writer.writeByte(item.getType());
		writer.writeFloat(item.getX());
		writer.writeFloat(item.getY());
		writer.writeFloat(item.getZ());
		writer.writeFloat(item.getYaw());
		writer.writeFloat(item.getHealth());
		writer.writeInt(item.getCount());
		writer.writeByte(item.getParentClass());
		writer.writeByte(item.getParentMode());
		writer.writeInt(item.getParentId());
	}

	private void writeUnits(S2MapUnits mapUnits) throws IOException {
		writer.writeInt(mapUnits.getCount());
		for (Iterator<Integer> i = mapUnits.getKeyIterator(); i.hasNext();) {
			S2Unit s2Unit = mapUnits.get(i.next());
			writeNextUnit(s2Unit);
		}
	}

	private void writeNextUnit(S2Unit unit) throws IOException {
		writer.writeInt(unit.getId());
		writer.writeByte(unit.getType());
		writer.writeFloat(unit.getX());
		writer.writeFloat(unit.getY());
		writer.writeFloat(unit.getZ());
		writer.writeFloat(unit.getYaw());
		writer.writeFloat(unit.getHealth());
		writer.writeFloat(unit.getHealthMax());
		writer.writeFloat(unit.getHunger());
		writer.writeFloat(unit.getThirst());
		writer.writeFloat(unit.getExhaustion());
		writer.writeFloat(unit.getAicx());
		writer.writeFloat(unit.getAicz());
	}

	private void writeObjects(S2MapObjects mapObjects) throws IOException {
		writer.writeInt(mapObjects.getCount());
		for (Iterator<Integer> i = mapObjects.getKeyIterator(); i.hasNext();) {
			S2Object s2Object = mapObjects.get(i.next());
			writeNextObject(s2Object);
		}
	}

	private void writeNextObject(S2Object object) throws IOException {
		writer.writeInt(object.getId());
		writer.writeByte(object.getType());
		writer.writeFloat(object.getX());
		writer.writeFloat(object.getZ());
		writer.writeFloat(object.getYaw());
		writer.writeFloat(object.getHealth());
		writer.writeFloat(object.getHealthMax());
		writer.writeInt(object.getDayTimer());
	}

	private void writeGrassmap(S2MapGrassmap grassmap) throws IOException {
		writer.write(grassmap.getData());
	}

	private void writeHeightmap(S2MapHeightmap heightmap) throws IOException {
		writer.writeInt(heightmap.getTerrainSize());
		writer.write(heightmap.getData());
	}

	private void writeColormap(S2MapColormap colormap) throws IOException {
		writer.writeInt(colormap.getSize());
		writer.write(colormap.getData());
	}

	private void writeMapInterface(S2MapInterface mapInterface) throws IOException {
		for (int i = 0; i < 10; i++)
			writer.writeString(mapInterface.getQuickslot(i));
	}

	private void writeMapVariables(S2MapVariables variables) throws IOException {
		writer.writeInt(variables.getTimeDay());
		writer.writeByte(variables.getTimeHour());
		writer.writeByte(variables.getTimeMinute());
		writer.writeByte(variables.getTimeFrozen());
		writer.writeString(variables.getSkybox());
		writer.writeByte(variables.getMultiplayer());
		writer.writeByte(variables.getClimate());
		writer.writeString(variables.getMusic());
		writer.writeString(variables.getBriefing());
		writer.writeByte(variables.getFogR());
		writer.writeByte(variables.getFogG());
		writer.writeByte(variables.getFogB());
		writer.writeByte(variables.getFogMode());
		writer.writeByte(variables.getExtendedStuff());
	}

	private void writePasswordHeader(S2MapPasswordHeader header) throws IOException {
		writer.writeByte(header.getCode());
		writer.writeLine(header.getPassword());
	}

	private void writeImage(S2MapImage image) throws IOException {
		writer.write(image.getData());
	}

	private void writeHeader(S2MapHeader header) throws IOException {
		writer.writeLine(header.getFirstLine());
		writer.writeLine(header.getVersion());
		writer.writeLine(header.getDate());
		writer.writeLine(header.getTime());
		writer.writeLine(header.getFormat());
		writer.writeLine(header.getMode());

		writer.writeLine("");
		
		writer.writeLine("");
		writer.writeLine("");
		writer.writeLine("");
		writer.writeLine("");
		writer.writeLine("###");
	}
}