package stranded2.map.io;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.ImageOutputStream;

public class RGBImage {

	private BufferedImage image;
	protected char[][][] rgbData;
	private char[] cbuf;

	public RGBImage(SimpleReader reader, int width, int height) throws IOException {
		image = new BufferedImage(width, height, BufferedImage.TYPE_INT_BGR);

		cbuf = new char[width * height * 3];
		reader.read(cbuf, 0, width * height * 3);

		rgbData = new char[height][width][3];

		int k = 0;
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				int r = rgbData[y][x][0] = cbuf[k++];
				int g = rgbData[y][x][1] = cbuf[k++];
				int b = rgbData[y][x][2] = cbuf[k++];
				int rgb = 255 * 256 * 256 * 256 + r * 256 * 256 + g * 256 + b;
				setImageData(x, y, rgb);
			}
		}

	}

	public RGBImage(String path, int width, int height) throws IOException {
		Iterator<ImageReader> imageReadersByFormatName = ImageIO.getImageReadersByFormatName("bmp");
		ImageReader reader = imageReadersByFormatName.next();
		ImageInputStream imageInputStream = ImageIO.createImageInputStream(new File(path));
		reader.setInput(imageInputStream);
		image = reader.read(0);
		imageInputStream.close();

		cbuf = new char[width * height * 3];
		rgbData = new char[height][width][3];

		int k = 0;
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				int argb = image.getRGB(x, y);
				byte[] array = new byte[4];
				ByteBuffer buffer = ByteBuffer.wrap(array);
				buffer.putInt(argb);
				int r = rgbData[y][x][0] = cbuf[k++] = (char) (array[1]<0?array[1]+256:array[1]);
				int g = rgbData[y][x][1] = cbuf[k++] = (char) (array[2]<0?array[2]+256:array[2]);
				int b = rgbData[y][x][2] = cbuf[k++] = (char) (array[3]<0?array[3]+256:array[3]);
				int rgb = 255 * 256 * 256 * 256 + r * 256 * 256 + g * 256 + b;
				setImageData(x, y, rgb);
			}
		}
	}

	public void setImageData(int x, int y, int rgb) {
		image.setRGB(x, y, rgb);
	}

	public char[] getData() {
		return cbuf;
	}

	public void createBitmap(String path) throws IOException {
		Iterator<ImageWriter> imageWritersByFormatName = ImageIO.getImageWritersByFormatName("bmp");
		ImageWriter writer = imageWritersByFormatName.next();
		ImageOutputStream imageOutputStream = ImageIO.createImageOutputStream(new File(path));
		writer.setOutput(imageOutputStream);
		writer.write(image);
		imageOutputStream.close();
	}
}
