package stranded2.map.io;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class SimpleWriter {

	File mapFile;
	FileOutputStream fileOutputStream;
	BufferedWriter writer;

	public SimpleWriter(String path) throws FileNotFoundException, UnsupportedEncodingException {
		mapFile = new File(path);
		fileOutputStream = new FileOutputStream(mapFile);
		writer = new BufferedWriter(new OutputStreamWriter(fileOutputStream, "ISO-8859-1"));
	}

	public void write(char[] cbuf) throws IOException {
		writer.write(cbuf);
	}

	public void writeLine(String s) throws IOException {
		if (s != null)
			writer.write(s);
		writer.newLine();
	}

	public void writeByte(Byte b) throws IOException {
		writer.write(b.byteValue()<0?b.byteValue()+256:b.byteValue());
	}

	public void writeInt(Integer integer) throws IOException {
		byte[] array = new byte[4];
		ByteBuffer wrap = ByteBuffer.wrap(array);
		wrap.order(ByteOrder.LITTLE_ENDIAN);
		wrap.putInt(integer);
		for (int k = 0; k < 4; k++) {
			writer.write(array[k]<0?array[k]+256:array[k]);
		}
	}

	public void writeFloat(Float f) throws IOException {
		byte[] array = new byte[4];
		ByteBuffer wrap = ByteBuffer.wrap(array);
		wrap.order(ByteOrder.LITTLE_ENDIAN);
		wrap.putFloat(f);
		for (int k = 0; k < 4; k++) {
			writer.write(array[k]<0?array[k]+256:array[k]);
		}
	}

	public void writeString(String s) throws IOException {
		int length = s.length();
		writeInt(length);
		writer.write(s);
	}

	public void write(byte[] data) throws IOException {
		for (int i = 0; i < data.length; i++)
			writeByte(data[i]);
	}

	public void close() throws IOException {
		writer.close();
		fileOutputStream.close();
	}
}